<?php

/*
 Plugin Name: iRobot.txt SEO
 Plugin URI: http://markbeljaars.com/plugins/irobottxt-seo/
 Description: iRobots.txt SEO is a SEO optimized, secure and customizable robots.txt virtual file creator.
 Version: 1.0.4
 Author: Mark Beljaars
 Author URI: http://markbeljaars.com
 */

/*
Copyright (C) 2009 Mark Beljaars, markbeljaars.com

This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 3 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program.  If not, see http://www.gnu.org/licenses/.
*/

define('IRSEO_VERSION', '1.0.4');
define('IRSEO_PLUGIN_PATH', 'wp-content/plugins/' . basename(dirname(__FILE__)));
define('IRSEO_SENTINEL', 'iRobots.txt SEO');

add_action('plugins_loaded','irseo_loaded_hook');
add_action('init', 'irseo_display_hook');

if ( is_admin() ) {
	add_action('admin_menu', 'irseo_menu_hook');
	add_filter('plugin_action_links','irseo_filter_settings_option', 10, 2);
	register_uninstall_hook(__FILE__, 'irseo_uninstall_hook');
}

function irseo_loaded_hook() {

    global $irseo_options;
    $irseo_options = array(
		'use_strict' => '',
		'allow_sitemap' => 'checked',
		'allow_adsense' => 'checked',
		'inhib_sys' => 'checked',
		'inhib_ia' => '',
		'inhib_img' => '',
		'inhib_dugg' => 'checked',
		'adv_config' => ''
		);
		
	foreach($irseo_options as $option => $value) {
		add_option('irseo_' . $option, $value);
	}
}

function irseo_uninstall_hook() {
	irseo_loaded_hook();

	foreach(array_keys($GLOBALS['irseo_options']) as $option) {
		delete_option('irseo_' . $option);
	}
}

function irseo_display_hook() {
	if(irseo_selfURL() != get_bloginfo('url') . '/robots.txt')	
		return;
	
	load_plugin_textdomain('irseo', constant('IRSEO_PLUGIN_PATH'), basename(dirname(__FILE__)));
	header('Content-type: text/plain');
	echo irseo_construct_robots();
	die;
}

function irseo_construct_robots() {

	$content = '#######################################################'. PHP_EOL
		  		. '# ' . constant('IRSEO_SENTINEL') . PHP_EOL;

	if(get_option('irseo_inhib_sys')) {
		$content .= PHP_EOL . '# All Bots' . PHP_EOL
					. 'User-agent: *' . PHP_EOL
					. 'Disallow: /cgi-bin' . PHP_EOL
					. 'Disallow: /wp-admin/' . PHP_EOL
					. 'Disallow: /wp-includes/' . PHP_EOL
					. 'Disallow: /wp-content/' . PHP_EOL
					. 'Disallow: /readme.html' . PHP_EOL
					. 'Disallow: /license.txt' . PHP_EOL
					. 'Disallow: /search/' . PHP_EOL;

		if(!get_option('irseo_use_strict')) {
			$content .= 'Disallow: /*?*' . PHP_EOL
						. 'Disallow: /*?' . PHP_EOL
						. 'Disallow: /*.php$' . PHP_EOL
						. 'Disallow: /*.js$' . PHP_EOL
						. 'Disallow: /*.inc$' . PHP_EOL
						. 'Disallow: /*.css$' . PHP_EOL
						. 'Disallow: /*.gz$' . PHP_EOL
						. 'Disallow: /*.wmv$' . PHP_EOL
						. 'Disallow: /*.cgi$' . PHP_EOL
						. 'Disallow: /*.xhtml$' . PHP_EOL
						. 'Allow: /wp-content/uploads/' . PHP_EOL
						. 'Allow: /*?page=*' . PHP_EOL;
		}
	}
	
	if(get_option('irseo_inhib_ia')) {
		$content .= PHP_EOL . '# Internet Archiver Wayback Machine' . PHP_EOL
					. 'User-agent: ia_archiver' . PHP_EOL
					. 'Disallow: /' . PHP_EOL;
	}

	if(get_option('irseo_inhib_img')) {
		$content .= PHP_EOL .'# Google Image' . PHP_EOL
					. 'User-agent: Googlebot-Image' . PHP_EOL
					. 'Disallow: /' . PHP_EOL
					. PHP_EOL . '# Yahoo Multimedia' . PHP_EOL
					. 'User-agent: Yahoo-MMCrawler' . PHP_EOL
					. 'Disallow: /' . PHP_EOL
					. PHP_EOL . '# Bing Picture Search' . PHP_EOL
					. 'User-agent: psbot' . PHP_EOL
					. 'Disallow: /' . PHP_EOL;
	}

	if(get_option('irseo_inhib_dugg')) {
		$content .= PHP_EOL .'# Dugg Mirror' . PHP_EOL
					. 'User-agent: duggmirror' . PHP_EOL
					. 'Disallow: /' . PHP_EOL;
	}

	if(get_option('irseo_allow_adsense')) {
		$content .= PHP_EOL .'# Google AdSense' . PHP_EOL
					. 'User-agent: Mediapartners-Google' . PHP_EOL
					. 'Disallow: ' . PHP_EOL;

		if(!get_option('irseo_use_strict')) {
			$content .= 'Allow: /' . PHP_EOL;
		}

	}
	
	$adv_options = get_option('irseo_adv_config');
	if($adv_options && is_array($adv_options)) {
		sort($adv_options);

		$content .= PHP_EOL .'# Custom Records';
		$last_useragent = '';
		foreach($adv_options as $option => $ua) {
		
			if(!get_option('irseo_use_strict') || ($ua['directive'] != 'Allow:'
						&& ($ua['userAgent'] == "*" || strpos($ua['userAgent'], '*') == FALSE) 
						&& strpos($ua['action'], '*') == FALSE && strpos($ua['action'], '$') == FALSE)) {
				if($ua['userAgent'] != $last_useragent) {
					$content .= PHP_EOL . 'User-agent: '. $ua['userAgent'] . PHP_EOL;
					$last_useragent = $ua['userAgent'];
				}
				$content .= $ua['directive'] . ' ' . $ua['action'] . PHP_EOL;
			}
		}
	}
	
	if(get_option('irseo_allow_sitemap')) {
		$content .= PHP_EOL . '# Sitemap' . PHP_EOL;
	
		if(get_option('irseo_use_strict')) {
			$content .= '# ' . __('Sitemaps not allowed if strict definition selected.', 'irseo');
		}
		
		elseif(irseo_file_exists(get_bloginfo('url') . '/sitemap.xml.gz')) {
			$content .= 'Sitemap: ' . get_bloginfo('url') . '/sitemap.xml.gz';
		}

		elseif(irseo_file_exists(get_bloginfo('url') . '/sitemap.xml')) {
			$content .= 'Sitemap: ' . get_bloginfo('url') . '/sitemap.xml';
		}

		else {
			$content .= __('# YOUR WEBSITE DOES NOT HAVE A SITEMAP! Please consider
# installing an automated sitemap generator such as
# Google XML Sitemaps -
# http://www.arnebrachhold.de/redir/sitemap-home/', 'irseo');
		}

		$content .= PHP_EOL;
	}
		
	$content .= PHP_EOL . '#######################################################' . PHP_EOL
			   . '#' . PHP_EOL
			   . '# Robots.txt ' . __('file generated by', 'irseo') . ' iRobots.txt SEO v' . constant('IRSEO_VERSION') . PHP_EOL
			   . '# by Mark Beljaars' . PHP_EOL
			   . '#' . PHP_EOL
			   . '#  _ _  _  _ |  |_  _ |. _  _  _ _  _ _  _ _ ' . PHP_EOL
			   . '# | | |(_||  |< |_)(/_||(_|(_|| _\.(_(_)| | |'. PHP_EOL
			   . '#                     _|'. PHP_EOL
			   . '# http://markbeljaars.com/plugins/irobotstxt-seo'. PHP_EOL
			   . '#'. PHP_EOL
			   . '#######################################################'. PHP_EOL;

	if(!get_option('irseo_use_strict')) {			   
		$content .= '#'. PHP_EOL
					. __('# Note:
# The Allow directive and wildcards (*) in filenames are
# not standard robots.txt syntax, however they are
# supported by most search engines.', 'irseo')
					. PHP_EOL;
	}

	return $content;
}

function irseo_physical_robots_exists() {
	$fp = @fopen(get_bloginfo('url') . '/robots.txt', 'r');
	if($fp !== false) {
		$content =  fread($fp, 1024);
		fclose($fp);
		return strpos($content, constant('IRSEO_SENTINEL')) === false;
	}
	return false;
}

function irseo_file_exists($url)
{
    $fp = @fopen($url, 'r');
    return ($fp !== false);   
} 

// The following function by S.M. Saidur Rahman
// http://ranawd.wordpress.com/2008/11/18/how-to-get-the-current-page-url/
function irseo_selfURL() {
	$s = empty($_SERVER['HTTPS']) ? '' : ($_SERVER["HTTPS"] == 'on') ? 's' : '';	
	$protocol = irseo_strleft(strtolower($_SERVER['SERVER_PROTOCOL']), '/').$s;
	$port = ($_SERVER['SERVER_PORT'] == '80') ? '' : (':'.$_SERVER['SERVER_PORT']);
	return $protocol.'://'.$_SERVER['SERVER_NAME'].$port.$_SERVER['REQUEST_URI'];
}

function irseo_strleft($s1, $s2) {
	$values = substr($s1, 0, strpos($s1, $s2));
	return  $values;
}

// The following function supplied by Jay Shergill (www.pdrater.com)
function irseo_filter_settings_option( $links, $file ) {
	static $this_plugin;
	
	if (!$this_plugin)
		$this_plugin = plugin_basename(__FILE__);

	if ($file == $this_plugin) {
		$settings_link = '<a href="options-general.php?page=irobotstxt-seo/irobotstxt-seo.php">' 
							. __('Settings', 'irseo') . '</a>';
		array_unshift($links, $settings_link);
	}
	return $links;
}
// ------------------------------------------------------------------------------

function irseo_menu_hook() {
	add_options_page('iRobots.txt SEO', 'iRobots.txt SEO', 8, __FILE__, 'irseo_plugin_options');
}

function irseo_admin_section_start($heading_text, $heading_name, $is_hidden = '1') {
	echo '<input type="hidden" id="' . $heading_name . '_display" name="' 
		. $heading_name . '_display" value="' . $_POST[$heading_name . '_display'] . '">' . PHP_EOL;
	if($_POST[$heading_name . '_display']) {
		$is_hidden = $_POST[$heading_name . '_display'] == 'none';
	}

	echo PHP_EOL . '<div id="normal-sortables" class="meta-box-sortables">'
		. '<div class="postbox">'
		. '<div class="handlediv" title="' . __('Click to toggle!', 'irseo') . '" onclick="toggleVisibility(\'' . $heading_name . '\');">'
		. '<br /></div><h3 class="hndle" title="' . __('Click to toggle!', 'irseo') 
		. '" onclick="toggleVisibility(\'' . $heading_name . '\');"><span>' . $heading_text . '</span></h3>'
		. '<div class="inside" id="' . $heading_name . '" style="display:' . ($is_hidden ? 'none' : 'block') . ';">' . PHP_EOL;
}

function irseo_admin_section_option($option_text, $tooltip_text, $option_name) {

	echo '<p class="meta-options">'
		. '<label for="' . $option_name . '" class="selectit">'
		. '<input name="' . $option_name . '" type="checkbox" id="'. $option_name .'"' 
		. (get_option($option_name) ? ' checked="checked"' : '') . ' />'
		. '&nbsp;' . $option_text
		. '</label>' . PHP_EOL;

	if($tooltip_text) {
		echo '<a title="' . __('Click here for help!', 'irseo') . '" onclick="toggleVisibility(\'' . $option_name . '_tip\');">'
			. '&nbsp;&nbsp;'
			. '<img border="0" src="' . get_bloginfo('url') . '/wp-content/plugins/' . basename(dirname(__FILE__)) . '/help.gif" />'
			. '</a>';
	}
	echo '</p>';
		
	if($tooltip_text) {
		echo '<div id="' . $option_name . '_tip" style="display:none;margin-left:8px;color:#21759b;">'
			. $tooltip_text
			. '<br /></div>' . PHP_EOL;
	}
}


function irseo_admin_section_end() {
	echo '</div></div></div>' . PHP_EOL;
}

function irseo_plugin_options() {

	load_plugin_textdomain('irseo', constant('IRSEO_PLUGIN_PATH'), basename(dirname(__FILE__)));

	echo '<script type="text/javascript">' . PHP_EOL 
		. '<!--' . PHP_EOL 
		. 'function toggleVisibility(id) { ' . PHP_EOL 
		. '   var e = document.getElementById(id);' . PHP_EOL 
		. '   if(e.style.display == "block")' . PHP_EOL 
		. '      e.style.display = "none";' . PHP_EOL 
		. '   else' . PHP_EOL 
		. '      e.style.display = "block";' . PHP_EOL 
		. '	document.getElementById(id.concat("_display")).value = e.style.display;' . PHP_EOL
		. '}' . PHP_EOL 
		. '-->' . PHP_EOL
		. '</script>' . PHP_EOL 
		. PHP_EOL;
		
	if (isset ($_POST['set_defaults'])) {
		foreach($GLOBALS['irseo_options'] as $option => $value) {
			update_option('irseo_' . $option, $value);
		}
		echo '<div id="message" class="updated fade"><p><strong>' . __('Default options loaded!', 'irseo') . '</strong></p></div>';
	}

	elseif (isset ($_POST['info_update'])) {
		foreach($GLOBALS['irseo_options'] as $option => $value) {
			if($option != 'adv_config') {
				update_option('irseo_' . $option, stripslashes((string) $_POST[('irseo_' . $option)]));
			}
		}
		echo '<div id="message" class="updated fade"><p><strong>' . __('Configuration updated!', 'irseo') . '</strong></p></div>';
	}
	
	elseif (isset ($_POST['adv_delete'])) {
		echo '<div id="message" class="updated fade">';

		$adv_options = get_option('irseo_adv_config');
		foreach($_POST['adv_delete'] as $element => $value) {
			unset($adv_options[$element]);
			echo '<p><strong>(ID ' . $element . ') ' . __('Custom record deleted!', 'irseo') . '</strong></p>';
		}

		update_option('irseo_adv_config', $adv_options);
		echo '</div>';
	}
	
	elseif (isset ($_POST['adv_update'])) {
		$adv_options = get_option('irseo_adv_config');
		
		if($_POST['irseo_adv_agent'] != '' && $_POST['irseo_adv_string'] != '') {
			$adv_options[] = array(
				"userAgent" => $_POST['irseo_adv_agent'],
				"directive" => $_POST['irseo_adv_directive'],
				"action" => $_POST['irseo_adv_string']);	

			update_option('irseo_adv_config', $adv_options);
			
			 end($adv_options);
			echo '<div id="message" class="updated fade"><p><strong>(ID ' . key($adv_options) . ') ' 
				. __('Custom record added!', 'irseo') . '</strong></p></div>';
		}
		else {
			echo '<div id="message" class="updated fade"><p><strong>' . __('Failed to add custom record!', 'irseo') . '</strong></p></div>';
		}
	}
	
	echo '<div class="wrap">'
		. '<h2>iRobots.txt SEO v' . constant('IRSEO_VERSION') . '</h2>'
		. '<div id="poststuff" class="metabox-holder has-right-sidebar">' . PHP_EOL;
	echo PHP_EOL . '<form method="post" action="" name="dofollow">' . PHP_EOL;

	if(irseo_physical_robots_exists()) {
		echo '<h4 style="text-align:center; color: #a00">' . __('WARNING: A PHYSICAL ROBOTS.TXT FILE ALREADY EXISTS!', 'irseo') . '</h4><p>' . PHP_EOL .
__('As iRobots.txt SEO only creates a virtual robots.txt file, it cannot replace a physical file. You may still use this plugin and view the results in the <em>View Robots.txt</em> window below, however the file will not be served until the physical file is deleted or renamed.', 'irseo')
			. '</p><br />' . PHP_EOL;
	}
	
	if(class_exists('GoogleSitemapGeneratorLoader')) {
		echo '<h4 style="text-align:center; color: #a00">' . __('XML SITEMAP GENERATOR PLUGIN DETECTED!', 'irseo') . '</h4><p>' . PHP_EOL .
__('The XML Sitemap Generator plugin creates a basic virtual robots.txt file that may conflict with IRSEO. Please ensure that the <em>Add sitemap URL to the virtual robots.txt file</em> option located in the <em>XML-Sitemap</em> administrator settings is de-selected.', 'irseo')
			. '</p><br />' . PHP_EOL;
	}	

	irseo_admin_section_start(__('General Options', 'irseo'), 'genopts', '');
	irseo_admin_section_option(__('Use strict robots.txt standard definition', 'irseo'), 
__('The official <a href="http://www.robotstxt.org/robotstxt.html" target="_blank">robots.txt definition</a> specifically identifies which directories or files a search engine can <strong>not</strong> index and does not include any directives for detailing which files a search engine can index. Google has expanded the definition to include an <code>allow</code> directive and also allows wildcards in file names. Although not officially supported, the ammended standard is understood by most search engines.', 'irseo')
		, 'irseo_use_strict');
	irseo_admin_section_option(__('Automatically add the website sitemap to the robots.txt file', 'irseo'), 
__('Sitemaps inform search engines of your site structure and also allow you to estimate how often your pages will change. Obviously search engines find this sort of information beneficial. The sitemap protocol is defined <a href="http://www.sitemaps.org/protocol.php" target="_blank">here</a>. Sitemaps can be automatically produced by Wordpress plugins such as <a href="http://www.arnebrachhold.de/projects/wordpress-plugins/google-xml-sitemaps-generator/" target="_blank">Google XML Sitemaps Generator</a>.', 'irseo')
		, 'irseo_allow_sitemap');
	irseo_admin_section_option(__('Inhibit indexing of Wordpress system folders', 'irseo'), 
__('Wordpress system folders such as the plugin and content directories are not keyword optimized and therefore should not be indexed by a search engine. Further, indexing system folders may present a security risk.', 'irseo')
		, 'irseo_inhib_sys');
	irseo_admin_section_option(__('Allow Google Adsense to access entire site', 'irseo'), 
__('<a href="https://www.google.com/adsense/" target="blank_">Google Adsense</a> automatically determines which ads are relevant for your audience by crawling the contents of your site. Giving Adsense full access to your site may result in more targeted advertisements. Ignore this option if you have not implemented Adsense.', 'irseo')
		, 'irseo_allow_adsense');
	irseo_admin_section_option(__('Inhibit indexing by the Internet Archive', 'irseo'), 
__('The <a href="http://www.archive.org/index.php" target="_blank">Internet Archive</a> is a not-for-profit organization with aims to archive all information on the Internet at regular intervals. It is speculated that Google uses the Internet Archive to determine the age of a website to assist in defining a site\'s authority. Some SEO experts recommend that the Internet Archive be disabled from indexing young website. The Internet Archive also raises issues of document control (old versions of your posts may be archived), intellectual property rights and privacy.', 'irseo')
		, 'irseo_inhib_ia');
	irseo_admin_section_option(__('Inhibit image indexing', 'irseo'), 
__('You may wish to inhibit search engines from indexing your images if your images are copyright, have been dubiously obtained (they infringe copyright), are not related to your site or are not likely to generate traffic. Affiliate marketers may also find that images may generate untargeted traffic thus affecting a site\'s conversion ratio.', 'irseo')
		, 'irseo_inhib_img');
	irseo_admin_section_option(__('Inhibit indexing by the Dugg Mirror', 'irseo'),
__('Duggmirror provides a mirror for the most popular stories on Digg.com. Sites are often overloaded by the amount of traffic Digg sends their way, causing the webpage to become unavailable. To alleviate the so-called "digg effect" Duggmirror hosts a mirror of the most popular stories making them available to Digg users. The problem is that Google may index the DuggMirror page before the source and inturn drive traffic from your site to the mirror.', 'irseo')
		, 'irseo_inhib_dugg');
	irseo_admin_section_end();

	irseo_admin_section_start(__('Advanced Configuration', 'irseo'), 'advops');
	echo '<div id="postcustomstuff">';
	
	echo '<table id="list-table">'
		. '<thead>'
		. '<tr>'
		. '<th class="left">' . __('User Agent', 'irseo') . '</th>'
		. '<th>' . __('Directive', 'irseo') . '</th>'
		. '<th>' . __('Action', 'irseo') . '</th>'
		. '<th>&nbsp;</th>'
		. '</tr>'
		. '</thead>'
		. '<tbody id="the-list" class="list:meta">';
	
	$adv_options = get_option('irseo_adv_config');
	if($adv_options && is_array($adv_options)) {	
		foreach($adv_options as $option => $ua) {
			echo '<tr><td><p>' . $ua['userAgent'] . '</p></td>'
				. '<td><p>' . $ua['directive'] .'</p></td>'
				. '<td><p>' . $ua['action'] . '</p></td>'
				. '<td class="submit">'
				. '<input style="margin:1px 0 0 0;" type="submit" name="adv_delete[' . $option . ']" value="' . __('Delete', 'irseo') . '" />' 
				. '</td></tr>';	
		}
	}

	echo '</tbody>'
		. '</table>';

	echo '<p><strong>' . __('Add a custom record:', 'irseo') . '</strong></p>';

	echo '<p><table>'
		. '<tr>'
		. '<td class="left" style="width:100px;"><label for="irseo_adv_agent"><p>' . __('User Agent:', 'irseo') . '</p></label></td>'
		. '<td><input type="text" id="irseo_adv_agent" name="irseo_adv_agent" value="" /></td>'
		. '</tr>'
		. '<tr>'
		. '<td><select name="irseo_adv_directive">' 
		. '<option value="Disallow:" selected="selected">Disallow:</option>' 
		. '<option value="Allow:">Allow:</option></select></td>'
		. '<td><input type="text" id="irseo_adv_string" name="irseo_adv_string" value="" /></td>'
		. '</tr>'
		. '<tr>'
		. '<td colspan="2">'
		. '<input type="submit" style="width:auto;" name="adv_update" value="' . __('Add Custom Record', 'irseo') . '" />' 
		. '</td>'
		. '</tr>'
		. '</table></p>' 
		. '</div>'
		. '<p>' . 
__('Custom records can be added or deleted from the robots.txt file using this form. A complete list of user agents can be found at <a href="http://www.user-agents.org/" target="_blank">http://www.user-agents.org/</a>. Examples of robot.txt directive strings (the text that goes after the <em>allow</em> or <em>disallow</em> directives) can be found at <a href="http://www.robotstxt.org/robotstxt.html" target="_blank">http://www.robotstxt.org/robotstxt.html</a>. Google\'s non-official extensions are described in detail in this <a href="http://googlewebmastercentral.blogspot.com/2008/06/improving-on-robots-exclusion-protocol.html" target="_blank">blog post</a>. Note that all <em>allow</em> directive records and directive strings including wildcard globbing will be ignored if <em>Use strict robots.txt standard definition</em> is selected.', 'irseo') 
		. '</p>';
	irseo_admin_section_end();

	irseo_admin_section_start(__('View Robots.txt', 'irseo'), 'viewrobots');
	echo '<div id="editorcontainer"><textarea rows="10" cols="40" name="content" id="content" readonly="readonly">'
		. irseo_construct_robots()
		. '</textarea></div>';
	irseo_admin_section_end();

	echo '<div>'
		. '<input type="submit" name="info_update" value="' . __('Update Options', 'irseo') . ' &raquo;" />'
		. '&nbsp;&nbsp'
		. '<input type="submit" style="background-Color:#E0FFFF;"'
		. 'onmouseover="this.style.backgroundColor=\'red\'" '
		. 'onmouseout="this.style.backgroundColor=\'#E0FFFF\'"'
		. 'name="set_defaults" value="' . __('Load Default Options', 'irseo') . ' &raquo;" />'
		. '</div>' . PHP_EOL
		. '</form></div></div>' . PHP_EOL;
}
?>
